class IgniteProperty {
    constructor() {
        this.callbacks = [];
        this._value = null;
    }

    get value() {
        return this._value;
    }

    set value(val) {
        var old = this._value;
        this._value = val;

        //Invoke any callbacks letting them know the value changed.
        for (var i = 0; i < this.callbacks.length; i++) {
            this.callbacks[i].invoke(old, val);
        }
    }

    attach(onChange) {
        var callback = new IgnitePropertyCallback(this, onChange);
        this.callbacks.push(callback);
        return callback;
    }
}

class IgnitePropertyCallback {
    constructor(property, callback) {
        this.callback = callback;
        this.property = property;
    }

    invoke(oldValue, newValue) {
        if (this.callback) {
            this.callback(oldValue, newValue);
        }
    }

    disconnect() {
        this.callback = null;

        if (this.property) {
            this.property.callbacks = this.property.callbacks.filter(callback => callback != this);
            this.property = null;
        }
    }
}

export {
    IgniteProperty
};